package org.eclipse.jdt.internal.compiler.ast;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import org.eclipse.jdt.core.compiler.CharOperation;
import org.eclipse.jdt.internal.compiler.ASTVisitor;
import org.eclipse.jdt.internal.compiler.ast.NumberLiteral;
import org.eclipse.jdt.internal.compiler.codegen.CodeStream;
import org.eclipse.jdt.internal.compiler.impl.FloatConstant;
import org.eclipse.jdt.internal.compiler.lookup.BlockScope;
import org.eclipse.jdt.internal.compiler.lookup.TypeBinding;
import org.eclipse.jdt.internal.compiler.util.FloatUtil;

public class FloatLiteral extends NumberLiteral {
  float value;
  
  public FloatLiteral(char[] token, int s, int e) {
    super(token, s, e);
  }
  
  public void computeConstant() {
    Float computedValue;
    boolean containsUnderscores = (CharOperation.indexOf('_', this.source) > 0);
    if (containsUnderscores)
      this.source = CharOperation.remove(this.source, '_'); 
    try {
      computedValue = Float.valueOf(String.valueOf(this.source));
    } catch (NumberFormatException numberFormatException) {
      try {
        float v = FloatUtil.valueOfHexFloatLiteral(this.source);
        if (v == Float.POSITIVE_INFINITY)
          return; 
        if (Float.isNaN(v))
          return; 
        this.value = v;
        this.constant = FloatConstant.fromValue(v);
      } catch (NumberFormatException numberFormatException1) {}
      return;
    } 
    float floatValue = computedValue.floatValue();
    if (floatValue > Float.MAX_VALUE)
      return; 
    if (floatValue < Float.MIN_VALUE) {
      boolean isHexaDecimal = false;
      for (int i = 0; i < this.source.length; i++) {
        switch (this.source[i]) {
          case '.':
          case '0':
            break;
          case 'X':
          case 'x':
            isHexaDecimal = true;
            break;
          case 'D':
          case 'E':
          case 'F':
          case 'd':
          case 'e':
          case 'f':
            if (isHexaDecimal)
              return; 
            break;
          case 'P':
          case 'p':
            break;
          default:
            return;
        } 
      } 
    } 
    this.value = floatValue;
    this.constant = FloatConstant.fromValue(this.value);
  }
  
  public void generateCode(BlockScope currentScope, CodeStream codeStream, boolean valueRequired) {
    int pc = codeStream.position;
    if (valueRequired)
      codeStream.generateConstant(this.constant, this.implicitConversion); 
    codeStream.recordPositionsFrom(pc, this.sourceStart);
  }
  
  public TypeBinding literalType(BlockScope scope) {
    return (TypeBinding)TypeBinding.FLOAT;
  }
  
  public void traverse(ASTVisitor visitor, BlockScope scope) {
    visitor.visit(this, scope);
    visitor.endVisit(this, scope);
  }
}
